@@ -30,7 +30,8 @@
 //! }
 //! let env = Salak::builder()
 //!     .set("config.val", "2021")
-//!     .unwrap_build();
+//!     .build()
+//!     .unwrap();
 //! let config = env.get::<Config>().unwrap();
 //! assert_eq!(2021, config.value);
 //! assert_eq!(None, config.optional);
@@ -40,7 +41,7 @@
 //! ## Features
 //!
 //! #### Predefined Sources
-//! Predefined sources has the following order, [`PropertyRegistry`] will find by sequence of these orders,
+//! Predefined sources has the following order, [`Salak`] will find by sequence of these orders,
 //! if the property with specified key is found at the current source, than return immediately. Otherwise,
 //! it will search the next source.
 //!
@@ -59,7 +60,7 @@
 //!    * `random.isize`
 //! 2. Custom arguments source. [`SalakBuilder::set()`] can set a single kv,
 //! and [`SalakBuilder::set_args()`] can set a group of kvs.
-//! 3. System environment source. Implemented by [`system_environment`].
+//! 3. System environment source. Implemented by [`source::system_environment`].
 //! 4. Profile specified file source, eg. `app-dev.toml`
 //! 5. Not profile file source, eg. `app.toml`
 //! 6. Custom sources, which can register by [`Salak::register()`].
@@ -114,39 +115,45 @@
     variant_size_differences
 )]
 
-#[cfg(feature = "derive")]
-#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
-mod derive;
+use std::sync::Mutex;
 
+#[cfg(feature = "derive")]
+use crate::derive::KeyDesc;
+#[cfg(feature = "derive")]
+mod derive;
 #[cfg(feature = "derive")]
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
-pub use crate::derive::{
-    AutoDeriveFromEnvironment, DescribableEnvironment, KeyDesc, PrefixedFromEnvironment,
-};
+pub use crate::derive::{AutoDeriveFromEnvironment, DescFromEnvironment, PrefixedFromEnvironment};
+use raw_ioref::IORefT;
 /// Auto derive [`FromEnvironment`] for struct.
 #[cfg(feature = "derive")]
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use salak_derive::FromEnvironment;
+use source_raw::PropertyRegistryInternal;
 
 #[cfg(feature = "args")]
 #[cfg_attr(docsrs, doc(cfg(feature = "args")))]
 mod args;
 #[cfg(feature = "args")]
 #[cfg_attr(docsrs, doc(cfg(feature = "args")))]
-pub use crate::args::*;
+pub use crate::args::AppInfo;
 
 mod err;
-pub use crate::raw::*;
 mod raw;
-pub use crate::env::*;
+use crate::raw::SubKey;
+pub use crate::raw::{IsProperty, Property};
+mod raw_ioref;
+mod raw_vec;
+use crate::env::PREFIX;
+pub use crate::env::{Salak, SalakBuilder};
 mod enums;
 mod env;
 
 pub use crate::enums::EnumProperty;
-pub use crate::err::{PropertyError, SalakParseError};
-pub use crate::source::{system_environment, HashMapSource, PropertyRegistry};
+pub use crate::err::PropertyError;
 
-mod source;
+mod source_map;
+mod source_raw;
 #[cfg(feature = "toml")]
 #[cfg_attr(docsrs, doc(cfg(feature = "toml")))]
 mod source_toml;
@@ -160,13 +167,34 @@ mod source_rand;
 #[cfg_attr(docsrs, doc(cfg(feature = "yaml")))]
 mod source_yaml;
 
+use crate::source::Key;
+use crate::source::SubKeys;
+
 #[cfg(test)]
 #[macro_use(quickcheck)]
 extern crate quickcheck_macros;
 
+/// Salak wrapper for configuration parsing.
+pub mod wrapper {
+    pub use crate::raw_ioref::IORef;
+    pub use crate::raw_vec::NonEmptyVec;
+}
+
+/// Salak property sources.
+pub mod source {
+
+    #[cfg(feature = "args")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "args")))]
+    pub(crate) use crate::args::from_args;
+    pub use crate::raw::Key;
+    pub use crate::raw::SubKeys;
+    pub use crate::source_map::system_environment;
+    pub use crate::source_map::HashMapSource;
+}
+
 /// An abstract source loader from various sources,
 /// such as command line arguments, system environment, files, etc.
-pub trait PropertySource {
+pub trait PropertySource: Send + Sync {
     /// [`PropertySource`] name.
     fn name(&self) -> &str;
 
@@ -174,38 +202,26 @@ pub trait PropertySource {
     fn get_property(&self, key: &Key<'_>) -> Option<Property<'_>>;
 
     /// Return next sub keys with prefix, sub keys are seperated by dot(.) in a key.
-    fn sub_keys<'a>(&'a self, key: &Key<'_>, sub_keys: &mut SubKeys<'a>);
+    fn get_sub_keys<'a>(&'a self, key: &Key<'_>, sub_keys: &mut SubKeys<'a>);
 
     /// Check whether the [`PropertySource`] is empty.
     /// Empty source will not be ignored when register to registry.
     fn is_empty(&self) -> bool;
 
-    #[doc(hidden)]
-    fn list_source_names<'a>(&'a self, names: &mut Vec<&'a str>) {
-        names.push(self.name());
+    /// Reload property source.
+    /// If nothing changes, then return none.
+    fn reload_source(&self) -> Result<Option<Box<dyn PropertySource>>, PropertyError> {
+        Ok(None)
     }
 }
 
 /// An environment for getting properties with mutiple [`PropertySource`]s, placeholder resolve and other features.
 pub trait Environment {
     /// Get config with specific type.
-    fn require<T: FromEnvironment>(&self, key: &str) -> Result<T, PropertyError> {
-        self.require_def(&mut Key::new(), SubKey::S(key), None)
-    }
+    fn require<T: FromEnvironment>(&self, key: &str) -> Result<T, PropertyError>;
 
-    /// Get config with specific type, if config not exists, then return default value.
-    /// * `key` - Property key.
-    /// * `sub_key` - Property sub key.
-    /// * `def` - Default property.
-    fn require_def<'a, T: FromEnvironment, K: Into<SubKey<'a>>>(
-        &'a self,
-        key: &mut Key<'a>,
-        sub_key: K,
-        def: Option<Property<'_>>,
-    ) -> Result<T, PropertyError>;
-
-    #[doc(hidden)]
-    fn sub_keys<'a>(&'a self, prefix: &Key<'_>, sub_keys: &mut SubKeys<'a>);
+    /// Reload configuration.
+    fn reload(&self) -> Result<bool, PropertyError>;
 
     #[cfg(feature = "derive")]
     #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
@@ -215,25 +231,49 @@ pub trait Environment {
     }
 }
 
-/// Convert from [`Environment`].
+/// Context for implementing [`FromEnvironment`].
+#[allow(missing_debug_implementations)]
+pub struct SalakContext<'a> {
+    registry: &'a PropertyRegistryInternal<'a>,
+    iorefs: &'a Mutex<Vec<Box<dyn IORefT + Send>>>,
+    key: &'a mut Key<'a>,
+}
+
+#[cfg(feature = "derive")]
+#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
+/// Context for implementing description.
+#[allow(missing_debug_implementations)]
+pub struct SalakDescContext<'a> {
+    key: &'a mut Key<'a>,
+    descs: &'a mut Vec<KeyDesc>,
+    current: KeyDesc,
+}
+
+/// Generate config from environment by [`SalakContext`].
 pub trait FromEnvironment: Sized {
-    /// Generate object from [`Environment`].
-    /// * `key` - Property key.
-    /// * `property` - Property value with key is `key`.
-    /// * `env` - Instance of [`Environment`]
-    fn from_env<'a>(
-        key: &mut Key<'a>,
+    /// Generate object from [`SalakContext`].
+    /// * `val` - Property value can be parsed from.
+    /// * `env` - Context.
+    ///
+    /// ```no_run
+    /// use salak::*;
+    /// pub struct Config {
+    ///   key: String
+    /// }
+    /// impl FromEnvironment for Config {
+    ///   fn from_env(
+    ///       val: Option<Property<'_>>,
+    ///       env: &mut SalakContext<'_>,
+    ///   ) -> Result<Self, PropertyError> {
+    ///     Ok(Self{
+    ///       key: env.require_def("key", None)?,
+    ///     })
+    ///   }
+    /// }
+    ///
+    /// ```
+    fn from_env(
         val: Option<Property<'_>>,
-        env: &'a impl Environment,
+        env: &mut SalakContext<'_>,
     ) -> Result<Self, PropertyError>;
-
-    #[cfg(feature = "derive")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
-    #[doc(hidden)]
-    fn key_desc<'a>(
-        key: &mut Key<'a>,
-        desc: &mut KeyDesc,
-        keys: &mut Vec<KeyDesc>,
-        env: &'a impl DescribableEnvironment,
-    );
 }
