@@ -16,6 +16,8 @@ pub enum Property<'a> {
     B(bool),
 }
 
+/// Any object implements this trait is automatically implmenting [`crate::FromEnvironment`].
+///
 /// This trait defines how to parse value from property, and defines specific behaviors such as
 /// how empty string being parsed.
 pub trait IsProperty: Sized {
@@ -166,13 +168,16 @@ impl IsProperty for Duration {
 
 /// Sub key is partial [`Key`] having values with either `[a-z][_a-z0-9]*` or [`usize`].
 #[derive(Debug)]
-pub enum SubKey<'a> {
+pub(crate) enum SubKey<'a> {
     /// Str sub key.
     S(&'a str),
     /// Index sub key.
     I(usize),
 }
 
+lazy_static::lazy_static! {
+    static ref P: &'static [char] = &['.', '[', ']'];
+}
 /// Key with a string buffer, can be avoid allocating memory when parsing configuration.
 #[derive(Debug)]
 pub struct Key<'a> {
@@ -180,10 +185,6 @@ pub struct Key<'a> {
     key: Vec<SubKey<'a>>,
 }
 
-lazy_static::lazy_static! {
-    static ref P: &'static [char] = &['.', '[', ']'];
-}
-
 impl<'a> Key<'a> {
     pub(crate) fn new() -> Self {
         Self {
@@ -250,13 +251,6 @@ impl<'a> Key<'a> {
     }
 }
 
-/// Sub key collection, which stands for lists of sub keys with same prefix.
-#[derive(Debug)]
-pub struct SubKeys<'a> {
-    pub(crate) keys: HashSet<&'a str>,
-    pub(crate) upper: Option<usize>,
-}
-
 impl<'a> From<&'a str> for SubKey<'a> {
     fn from(mut u: &'a str) -> Self {
         if u.starts_with('[') {
@@ -280,10 +274,16 @@ impl From<usize> for SubKey<'_> {
         SubKey::I(u)
     }
 }
+/// Sub key collection, which stands for lists of sub keys with same prefix.
+#[derive(Debug)]
+pub struct SubKeys<'a> {
+    keys: HashSet<&'a str>,
+    upper: Option<usize>,
+}
 
 impl<'a> SubKeys<'a> {
     /// Insert a sub key.
-    pub fn insert<K: Into<SubKey<'a>>>(&mut self, key: K) {
+    pub(crate) fn insert<K: Into<SubKey<'a>>>(&mut self, key: K) {
         match key.into() {
             SubKey::S(s) => {
                 self.keys.insert(s);
@@ -298,6 +298,31 @@ impl<'a> SubKeys<'a> {
             }
         }
     }
+
+    pub(crate) fn str_keys(&self) -> Vec<&'a str> {
+        self.keys
+            .iter()
+            .filter(|a| {
+                if let Some(c) = a.chars().next() {
+                    c < '0' && c > '9'
+                } else {
+                    false
+                }
+            })
+            .copied()
+            .collect()
+    }
+
+    pub(crate) fn new() -> Self {
+        Self {
+            keys: HashSet::new(),
+            upper: None,
+        }
+    }
+
+    pub(crate) fn max(&self) -> Option<usize> {
+        self.upper
+    }
 }
 
 #[cfg(test)]
@@ -593,25 +618,21 @@ mod tests {
     }
 
     impl FromEnvironment for Config {
-        fn from_env<'a>(
-            key: &mut Key<'a>,
+        fn from_env(
             _: Option<Property<'_>>,
-            env: &'a impl Environment,
+            env: &mut SalakContext<'_>,
         ) -> Result<Self, PropertyError> {
             Ok(Config {
-                i8: env.require_def(key, SubKey::S("i8"), None)?,
+                i8: env.require_def("i8", None)?,
             })
         }
+    }
 
-        #[cfg(feature = "derive")]
-        #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
-        fn key_desc<'a>(
-            key: &mut Key<'a>,
-            _: &mut KeyDesc,
-            keys: &mut Vec<KeyDesc>,
-            env: &'a impl DescribableEnvironment,
-        ) {
-            env.key_desc::<i8, &str>(key, "i8", None, None, None, keys);
+    #[cfg(feature = "derive")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
+    impl DescFromEnvironment for Config {
+        fn key_desc(env: &mut SalakDescContext<'_>) {
+            env.add_key_desc::<i8>("i8", None, None, None);
         }
     }
     #[test]
